#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Alexander Rind & the SoniVis team.
# Distributed under the terms of the MIT License (see LICENSE.txt).

"""
Information about the frontend package of the widgets.
"""

module_name = "sonivis-lens-widget"
module_version = "^0.1.0"
