#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Alexander Rind & the SoniVis team.
# Distributed under the terms of the MIT License (see LICENSE.txt).

version_info = (0, 1, 0, 'dev')
__version__ = ".".join(map(str, version_info))
